@@ -1,6 +1,3 @@
-## Log likelihood function included here because it uses output from getTheta
-getLoglik <- function(ySptIndex, fTilt) sum(log(fTilt[cbind(ySptIndex, seq_along(ySptIndex))]))
-
 ## Computes log(sum(exp(x))) with better precision
 logSumExp <- function(x)
 {
@@ -50,17 +47,44 @@ theta.control <- function(eps=1e-10, maxiter=100, maxhalf=20, maxtheta=500,
 #' @param f0 Values of the baseline distribution corresponding to the values of spt
 #' @param mu The fitted mean for each observation. Note these values must lie
 #' strictly within the range of the support.
+#' @param sampprobs Matrix of sampling probabilities. The number of rows should
+#' equal the number of observations, and the number of columns should equal
+#' the number of unique observed support points.
+#' @param ySptIndex Vector containing index of each obervation's response value
+#' within the \code{spt} vector. This is only needed to calculate the log-likelihood
+#' after each update.
 #' @param thetaStart Vector of starting values. One value per observation. If
 #' \code{NULL}, zero is used as the starting value for each observation.
 #' @param thetaControl Object of class \code{thetaControl}, which is a list of
 #' control arguments returned by the \code{thetaControl} function.
 #'
-#' @return List containing theta, b (sum(exp(theta*spt))), bPrime (the fitted
-#' means, which should match mu), bPrime2 (the fitted variances), and tiltFact
-#' (sptN x n matrix).
+#' @return List containing the following:
+#' \itemize{
+#' \item \code{theta} Updated values.
+#' \item \code{fTilt} Matrix containing the exponentially tilted distribution for each
+#' observation, i.e. f(y|X=x). Each column corresponds to an observation and sums to one.
+#' \item \code{bPrime} Vector containing the mean of the exponentially tilted distribution
+#' for each observation. Should match \code{mu} argument very closely.
+#' \item \code{bPrime2} Vector containing the variance of the exponentially tilted
+#' distribution for each observation.
+#' \item \code{fTiltSW} Matrix containing the exponentially tilted distribution for each
+#' observation, conditional on that observation being sampled, i.e. f(y|X=x, S=1).
+#' If \code{sampprobs=NULL}, then \code{fTiltSW} matches \code{fTilt}.
+#' \item \code{bPrimeSW} Vector containing the mean for each observation, conditional
+#' on that observation being sampled. If \code{sampprobs=NULL}, then \code{bPrimeSW}
+#' matches \code{bPrime}.
+#' \item \code{bPrime2SW} Vector containing the variance for each observation, conditional
+#' on that observation being sampled. If \code{sampprobs=NULL}, then \code{bPrime2SW}
+#' matches \code{bPrime2}.
+#' \item \code{llik} Semiparametric log-likelihood, evaluated at the current beta
+#' and f0 values. If sampling weights are used, then the log-likelihood is conditional
+#' on each observation being sampled.
+#' \item \code{conv} Convergence indicator.
+#' \item \code{iter} Number of iterations until convergence was reached.
+#' }
 #'
 #' @keywords internal
-getTheta <- function(spt, f0, mu, thetaStart=NULL, thetaControl=theta.control())
+getTheta <- function(spt, f0, mu, sampprobs, ySptIndex, thetaStart=NULL, thetaControl=theta.control())
 {
     ## Extract control arguments
     if (class(thetaControl) != "thetaControl")
@@ -189,5 +213,22 @@ getTheta <- function(spt, f0, mu, thetaStart=NULL, thetaControl=theta.control())
         s <- s[!conv[s] & !maxedOut[s]]
     }
 
-    list(theta=theta, bPrime=bPrime, bPrime2=bPrime2, fTilt=fTilt, conv=conv, iter=iter)
+    ## Obtain conditional distribution of sampled observations, if sampling weights are included
+    if (!(is.null(sampprobs))) {
+        fTiltSW <- fTilt * t(sampprobs)
+        fTiltSW <- fTiltSW / rep(colSums(fTiltSW), each=nrow(fTiltSW))
+        bPrimeSW <- colSums(spt * fTiltSW)
+        bPrime2SW <- colSums(outer(spt, bPrimeSW, "-")^2 * fTiltSW)
+    } else {
+        fTiltSW <- fTilt
+        bPrimeSW <- bPrime
+        bPrime2SW <- bPrime2
+    }
+
+    ## Calculate log-likelihood
+    llik <- sum(log(fTiltSW[cbind(ySptIndex, seq_along(ySptIndex))]))
+
+    list(theta=theta, fTilt=fTilt, bPrime=bPrime, bPrime2=bPrime2,
+         fTiltSW=fTiltSW, bPrimeSW=bPrimeSW, bPrime2SW=bPrime2SW,
+         llik=llik, conv=conv, iter=iter)
 }
