@@ -1,4 +1,4 @@
-test_that("getTheta does not return correct theta.", {
+test_that("getTheta returns correct theta.", {
     set.seed(100)
     n <- 100
     sptN <- 10
@@ -6,6 +6,6 @@ test_that("getTheta does not return correct theta.", {
     spt <- seq(-sptMax, sptMax, length.out=sptN)
     f0 <- rep(1/sptN, sptN)
     mu <- seq(-sptMax, sptMax, length.out=n)  # includes mu on boundary on support
-    th <- gldrm:::getTheta(spt, f0, mu)
+    th <- gldrm:::getTheta(spt=spt, f0=f0, mu=mu, sampprobs=NULL, ySptIndex=1, thetaStart=NULL)
     expect_equal(mu, th$bPrime)
 })
